#include "precompile.h"

#include <google/protobuf/message.h>
#include <a8/mixedsession.h>
#include <a8/tcplistener.h>

#include <f8/netmsghandler.h>
#include <f8/msgqueue.h>

#include "app.h"
#include "GGListener.h"
#include "jsondatamgr.h"
#include "handlermgr.h"

class GGClientSession: public a8::MixedSession
{
public:

    virtual void DecodeUserPacket(char* buf, int& offset, unsigned int buflen) override
    {
        #if 1
        is_activite = true;
        #endif
        bool warning = false;
        while (buflen - offset >= sizeof(f8::PackHead)) {
            f8::PackHead* p = (f8::PackHead*)&buf[offset];
            if (p->magic_code == f8::MAGIC_CODE) {
                if (buflen - offset < sizeof(f8::PackHead) + p->packlen) {
                    break;
                }
                f8::App::Instance()->AddSocketMsg
                    (SF_GameGate,
                     socket_handle,
                     saddr,
                     p->msgid,
                     p->seqid,
                     &buf[offset + sizeof(f8::PackHead)],
                     p->packlen,
                     0);
                offset += sizeof(f8::PackHead) + p->packlen;
            } else {
                warning = true;
                offset++;
                continue;
            }
        }

        if (warning) {
            f8::UdpLog::Instance()->Warning("收到client非法数据包", {});
        }
    }

    virtual void OnRawHttpGet(const std::string& url, const std::string& querystr,
                              std::string& response) override
    {
        f8::MsgQueue::Instance()->PostMsg
            (IM_ExecGM,
             a8::Args
             (
              {
               socket_handle,
               a8::XValue(url).GetString(),
               a8::XValue(querystr).GetString(),
               saddr
              }
              ));
    }

    virtual void OnDisConnect() override
    {
        f8::MsgQueue::Instance()->PostMsg
            (IM_ClientSocketDisconnect,
             a8::Args
             (
              {
               socket_handle
              }
              ));
    }

};

static void GSListeneron_error(a8::TcpListener*, int type, int errorid)
{
    f8::UdpLog::Instance()->Debug("GGListeneron_error %d %d", {type, errorid});
}

void GGListener::Init()
{
    tcp_listener_ = std::make_shared<a8::TcpListener>();
    tcp_listener_->RegisterSessionClass<GGClientSession>(1024 * 64);
    tcp_listener_->on_error = GSListeneron_error;

    tcp_listener_->bind_address = "0.0.0.0";
    tcp_listener_->bind_port = JsonDataMgr::Instance()->GetConf()->At("listen_port")->AsXValue();
    tcp_listener_->Open();
}

void GGListener::UnInit()
{
}

void GGListener::SendText(int sockhandle, const std::string& text)
{
    tcp_listener_->SendClientMsg(sockhandle, text.data(), text.size());
}

void GGListener::ForceCloseClient(int sockhandle)
{
    tcp_listener_->ForceCloseClient(sockhandle);
}

void GGListener::MarkClient(int sockhandle, bool is_active)
{
    tcp_listener_->MarkClient(sockhandle, is_active);
}
