#pragma once

#include <f8/timer.h>

namespace ss
{
    class SS_WSP_RequestTargetServer;
    class SS_WSP_HttpTunnelRequest;
    class SS_Ping;
}

struct GSNode;
class GSMgr : public a8::Singleton<GSMgr>
{
 public:
    enum { HID = HID_GSMgr };

 private:
    GSMgr() {};
    friend class a8::Singleton<GSMgr>;

 public:

    void Init();
    void UnInit();

    void _SS_WSP_RequestTargetServer(f8::MsgHdr* hdr, const ss::SS_WSP_RequestTargetServer& msg);
    void _SS_WSP_HttpTunnelRequest(f8::MsgHdr* hdr, const ss::SS_WSP_HttpTunnelRequest& msg);
    void _SS_Ping(f8::MsgHdr* hdr, const ss::SS_Ping& msg);

    void ___GSReport(std::shared_ptr<f8::JsonHttpRequest> request);
    void ___GSList(std::shared_ptr<f8::JsonHttpRequest> request);

 private:
    std::shared_ptr<GSNode> GetNodeByTeamId(const std::string& team_id);
    std::shared_ptr<GSNode> GetNodeByNodeKey(const std::string& node_key);
    std::shared_ptr<GSNode> GetNodeByInstanceId(int instance_id);
    std::shared_ptr<GSNode> AllocNode();
    void RearrangeNode();
    void AddNodeToSortedNodes(std::shared_ptr<GSNode> node);
    void RemoveNodeFromSortedNodes(std::shared_ptr<GSNode> node);
    std::vector<std::shared_ptr<GSNode>>* GetSortedNodes();
    void OnVersionChange(std::shared_ptr<GSNode> node);

 private:
    f8::Attacher timer_attacher_;
    std::map<std::string, std::shared_ptr<GSNode>> team_hash_;
    std::map<std::string, std::shared_ptr<GSNode>> node_key_hash_;
    std::map<int, std::vector<std::shared_ptr<GSNode>>> sorted_node_hash_;
};
